@@ -174,6 +174,7 @@ void empathy_account_settings_apply_async (EmpathyAccountSettings *settings,
 gboolean empathy_account_settings_apply_finish (
   EmpathyAccountSettings *settings,
   GAsyncResult *result,
+  gboolean *reconnect_required,
   GError **error);
 
 void empathy_account_settings_set_regex (EmpathyAccountSettings *settings,
@@ -189,6 +190,8 @@ gboolean empathy_account_settings_is_valid (EmpathyAccountSettings *settings);
 const TpConnectionManagerProtocol * empathy_account_settings_get_tp_protocol (
     EmpathyAccountSettings *settings);
 
+gboolean empathy_account_settings_supports_sasl (EmpathyAccountSettings *self);
+
 G_END_DECLS
 
 #endif /* #ifndef __EMPATHY_ACCOUNT_SETTINGS_H__*/
